/* --------------------------------------------------------------
This applet imitates a Coke Machine using a Finite State Machine,
Technically, the machine in this applet is a Moore automaton.

The input alphabet is {NICKLE, DIME, QUARTER}.
The states are the set {0, . . . , 14}.
F is the state transition function, and G is the output function.

-------------------------------------------------------------- */

import java.applet.Applet;
import java.awt.*;

public class CokeMachine extends Applet {
   public static final int NICKLE = 0, DIME = 1, QUARTER = 2;

   // The current state of the machine.
   private int state = 0;
   
   // F is the state transition function.
   private int[][] F = {{ 1,  2,  5}, { 2,  3,  6}, { 3,  4,  7},
                        { 4,  5,  8}, { 5,  6,  9}, { 6,  7, 10},
                        { 7,  8, 11}, { 8,  9, 12}, { 9, 10, 13},
                        {10, 11, 14}, { 1,  2,  5}, { 1,  2,  5},
                        { 1,  2,  5}, { 1,  2,  5}, { 1,  2,  5}};
   
   // G is the output function.
   private String[] G = {" . . . . . . . ", " . . . . . . . ", " . . . . . . . ",
                         " . . . . . . . ", " . . . . . . . ", " . . . . . . . ",
                         " . . . . . . . ", " . . . . . . . ", " . . . . . . . ",
                         " . . . . . . . ", " Coke          ", " Coke + $0.05  ",
                         " Coke + $0.10  ", " Coke + $0.15  ", " Coke + $0.20  "};
   
   private void dropIn(int coin) {
   
      state = F[state][coin];
      myState.setText("  State = " + state + "  ");
      output.setText(G[state]);
   
   }
   

   /** Add a coin to the machine. */
   public boolean action(Event e, Object arg) {
   
      System.out.println("Action Event");
	  if (e.target == nickleButton) {
         dropIn( NICKLE );
	  } else if (e.target == dimeButton) {
         dropIn( DIME );
	  } else if (e.target == quarterButton) {
         dropIn( QUARTER );
	  }
      repaint();
      
      return true;
   }
	
   private Label   myName, myState, output;
   private Button  nickleButton, dimeButton, quarterButton;
   private Panel   buttonPanel, statePanel, outputPanel;
   private Font    theFont;
   
   /** Create the GUI. */
   public void init() {

      this.setLayout(new GridLayout(4, 1, 10, 10));
      this.setBackground(Color.red);
      
      theFont = new Font("sansserif", Font.BOLD, 24);
      myName = new Label("      Coke Machine      ", Label.CENTER);
      myName.setFont( theFont );
      this.add( myName );
      
      statePanel = new Panel();
      theFont = new Font("monospaced", Font.PLAIN, 18);
      myState = new Label("  State =  0  ", Label.CENTER);
      myState.setFont( theFont );
      myState.setBackground(Color.white);
      statePanel.add( myState );
      this.add( statePanel );
      
      buttonPanel = new Panel();
      nickleButton = new Button("N");
      buttonPanel.add( nickleButton );
      dimeButton = new Button("D");
      buttonPanel.add( dimeButton );
      quarterButton = new Button("Q");
      buttonPanel.add( quarterButton );
      this.add( buttonPanel );

      outputPanel = new Panel();
      theFont = new Font("monospaced", Font.BOLD, 18);
      output = new Label(" . . . . . . . ", Label.CENTER);
      output.setFont( theFont );
      output.setBackground(Color.white);
      outputPanel.add( output );
      this.add( outputPanel );

   }
   
}
