import java.util.*;
import java.lang.reflect.Array;

abstract public class M865PriorityQueue implements Cloneable, Collection {
	protected long hash;  //  The cached hash code.
	
	/* This abstract class assumes that enqueue, dequeue, clear, and
		clone will properly update the hash code for the queue.  The
		chached value of the code should be used by iterators to check
		for any changes in the queue while the iterator is being used.
	*/
	
	abstract public void enqueue(Prioritizeable p);
	// Adds a Prioritizeable to the priority queue.

	abstract public Prioritizeable dequeue();
	// Removes and returns a Prioritizeable with the highest priority from
	// the priority queue.
	// Given two objects of equal priority, there is no guarantee as to which
	// object will be returned.

	abstract public Prioritizeable peek();
	// Returns an Prioritizeable with the highest priority from the queue.
	
	abstract public void adjustPriority(Prioritizeable obj, Comparable priority);
	//  Finds the object and adjusts its priority.
	
	
	public boolean add(Object obj) {
	// Ensures that this collection contains the specified element (optional operation).
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Prioritizeable)) {
			throw( new ClassCastException("Can only add Prioritizeables"));
		}
		enqueue((Prioritizeable)obj);
		return true;
	}
	
	public boolean addAll(Collection c) {
	// Adds all of the elements in the specified collection to this stack
		Iterator ci = c.iterator();
		boolean changed = ci.hasNext();
		while (ci.hasNext()) {
			this.add( ci.next() );
		}
		return changed;
	}

	public boolean contains(Object obj) {
	// Returns true if this collection contains the specified object.  
		boolean b = false;
		Iterator iter = this.iterator();
		while (!b && iter.hasNext()) {
			b = obj.equals( iter.next() );
		}
		return b;
	}

	public boolean containsAll(Collection c) {
	// Returns true if this collection contains all of the elements in the
	// specified collection. 
		Iterator ci = c.iterator();
		boolean b = true;
		while (b && ci.hasNext()) {
			b = this.contains(ci.next());
		}
		return b;
	}

	public boolean equals(Object o) {
	// Compares the specified object with this collection for equality. 
		boolean b = (o instanceof M865PriorityQueue) && (this.size() == ((M865PriorityQueue)o).size());
		if (b) {
			Iterator i = this.iterator();
			Iterator j = ((M865PriorityQueue)o).iterator();
			while (b && i.hasNext()) {
				b = (i.next()).equals(j.next());
			}
		}
		return b;
	}

	public int hashCode() {
	// Returns the hash code value for this collection.  
	// Note - this routine should return the same hash code for any two
	// M865PriorityQueues that have the same objects stored in the same order
	// The hash code should be updated within the enqueue and dequeue
	// methods
		return ( (int)hash );
	}

	public boolean isEmpty() {
	// Returns true if this collection contains no elements.  
		return (this.size() == 0);
	}
	 	 
	public boolean removeAll(Collection c) {
	// Each element in the specified collection is removed from the priority queue.
		Iterator ci = c.iterator();
		boolean b = false;
		while (ci.hasNext()) {
			b = b || this.remove(ci.next());
		}
		return b;
	}
	 
	public boolean retainAll(Collection c) {
	// Retains only the elements in this collection that are contained in the
	// specified collection (optional operation).		
		throw( new UnsupportedOperationException("retainAll is not implemented for the general M865PriorityQueue"));
	}

	public Object[] toArray() {
	// Returns an array containing all of the elements in this collection.
	// The elements in the array will be in their priority order
		Object[] newArray = new Object[this.size()];
		Iterator iter = this.iterator();
		int k=0;
		while (iter.hasNext()) {
			newArray[k++] = iter.next();
		}
		return newArray;
	}
	
	public Object[] toArray(Object[] a) {
	// Returns an array containing all of the elements in this collection; 
	//the runtime type of the returned array is that of the specified array.
		Iterator iter = this.iterator();
		int k = 0;
		if (this.size() <= a.length) {
			while (iter.hasNext()) {
				a[k++] = iter.next();
			}
			while (k < a.length) {
				a[k++] = null;
			}
			return a;
		} else {
			Class c = a.getClass();
			c = c.getComponentType();
			Object[] newArray = (Object[])Array.newInstance(c, this.size());
			while (iter.hasNext()) {
				newArray[k++] = iter.next();
			}
			return newArray;
		}
	}
	
	public String toString() {
		String s = "M865PriorityQueue {";
		Iterator iter = this.iterator();
		while (iter.hasNext()) {
			s += "\n " + iter.next().toString();
		}
		return (s + "\n}");
	}
	
}
