import java.util.*;
import java.lang.reflect.Array;

abstract public class M865Queue implements Cloneable, Collection {
	protected long hash;  //  The cached hash code.
   
   /* This abstract class assumes that enqueue, dequeue, clear, and
		clone will properly update the hash code for the queue.  The
		chached value of the code should be used by iterators to check
		for any changes in the queue while the iterator is being used.
   */
   
   abstract public void enqueue(Object o);
   //  appends an object to the end of the queue.
   
   abstract public Object dequeue();
   //  removes and returns the object at the beginning of the queue.
   //  returns null if the queue is empty.
      
   abstract public Object peek();
   //  returns the object at the beginning of the queue.
   
	public boolean add(Object o) {
	// Ensures that this collection contains the specified element.
		this.enqueue( o );
		return true;
	}
	
	public boolean addAll(Collection c) {
	// Adds all of the elements in the specified collection to this stack
		Iterator ci = c.iterator();
		boolean changed = ci.hasNext();
		while (ci.hasNext()) {
			this.enqueue( ci.next() );
		}
		return changed;
	}
	
	public boolean contains(Object o) {
	// Returns true if this collection contains the specified element.  
		boolean b = false;
		Iterator iter = this.iterator();
		while (!b && iter.hasNext()) {
			b = o.equals(iter.next());
		}
		return b;
	}
	
	public boolean containsAll(Collection c) {
	// Returns true if this collection contains all of the elements in the
	// specified collection. 
		Iterator ci = c.iterator();
		boolean b = true;
		while (b && ci.hasNext()) {
			b = this.contains(ci.next());
		}
		return b;
	}
	
	public boolean equals(Object o) {
	// Compares the specified object with this collection for equality. 
		boolean b = (o instanceof M865Queue) && (this.size() == ((M865Queue)o).size());
		if (b) {
			Iterator i = this.iterator();
			Iterator j = ((M865Queue)o).iterator();
			while (b && i.hasNext()) {
				b = (i.next()).equals(j.next());
			}
		}
		return b;
	}

	protected void updateHashCode(Object obj) {
		hash = (hash + obj.hashCode()) % Integer.MAX_VALUE;
	}
	
	protected void downdateHashCode(Object obj) {
		hash = (hash - obj.hashCode()) % Integer.MAX_VALUE;
	}	

	public int hashCode() {
	// Returns the hash code value for this collection.  
	// Note - this routine should return the same hash code for any two
	// M865Queues that have the same objects stored in the same order
	// The hash code should be updated within the enqueue and dequeue
	// methods
		return ( (int)hash );
	}

	public boolean isEmpty() {
	// Returns true if this collection contains no elements.  
		return (this.size() == 0);
	}

	public boolean remove(Object o) {
	// Removes a single instance of the specified element from this collection,
	// if it is present (optional operation). 
		throw( new UnsupportedOperationException("remove is not implemented for the general M865Queue"));
	}
	 	 
	public boolean removeAll(Collection c) {
	// Removes all this collection's elements that are also contained in the
	// specified collection (optional operation). 
		throw( new UnsupportedOperationException("removeAll is not implemented for the general M865Queue"));
	}
	 
	public boolean retainAll(Collection c) {
	// Retains only the elements in this collection that are contained in the
	// specified collection (optional operation).		
		throw( new UnsupportedOperationException("retainAll is not implemented for the general M865Queue"));
	}

	public Object[] toArray() {
	// Returns an array containing all of the elements in this collection.
		Object[] newArray = new Object[this.size()];
		Iterator iter = this.iterator();
		int k=0;
		while (iter.hasNext()) {
			newArray[k++] = iter.next();
		}
		return newArray;
	}
	
	public Object[] toArray(Object[] a) {
	// Returns an array containing all of the elements in this collection; 
	//the runtime type of the returned array is that of the specified array.
		Iterator iter = this.iterator();
		int k = 0;
		if (this.size() <= a.length) {
			while (iter.hasNext()) {
				a[k++] = iter.next();
			}
			while (k < a.length) {
				a[k++] = null;
			}
			return a;
		} else {
			Class c = a.getClass();
			c = c.getComponentType();
			Object[] newArray = (Object[])Array.newInstance(c, this.size());
			while (iter.hasNext()) {
				newArray[k++] = iter.next();
			}
			return newArray;
		}
	}
	
	public String toString() {
		String s = "M865Queue {";
		Iterator i = this.iterator();
		while (i.hasNext()) {
			s += "\n	" + i.next().toString();
		}
		return (s + "\n}");
	}
	
}
