public class PrioritizedItem implements Prioritizeable {
	protected Comparable priority;
	protected Object datum;
	
	protected PrioritizedItem(Comparable p, Object obj) {
		priority = p;
		datum =obj;
	}
	
	protected PrioritizedItem() {
		priority = null;
		datum = null;
	}
	
	public Object getDatum() {
		return datum;
	}
	
	public Comparable getPriority() {
		return priority;
	}

	public void setDatum(Object obj) {
		datum = obj;
	}
			
	public void setPriority(Comparable p) {
		priority = p;
	}
	
	public boolean adjustPriority(Comparable p) {
		Comparable oldPriority = priority;
		priority = p;
		return ( priority.compareTo(oldPriority) <= 0 );
	}

	public boolean higherThan(Prioritizeable obj) {
	// Returns true if this object has priority equal to or higher than the
	// specified object.  If the objects are not comparable, then the method
	// may throw a ClassCastException.
		return (this.compareTo( obj ) <= 0);
	}

	public boolean lowerThan(Prioritizeable obj) {
	// Returns true if this object has priority strictly lower than the
	// specified object.  If the objects are not comparable, then the method
	// may throw a ClassCastException.
		return (0 < this.compareTo( obj ));
	}
	
	
	public int compareTo(Object pqi) {
		if (pqi instanceof PrioritizedItem) {
			return priority.compareTo( ((PrioritizedItem)pqi).getPriority() );
		} else {
			throw( new ClassCastException("Not a PrioritizedItem") );
		}
	}


	public Object clone() {
	// Returns a copy of this queue
		return (new PrioritizedItem(priority, datum));
	}

	public boolean equals(Object obj) {
		boolean b = obj instanceof PrioritizedItem;
		if ( b ) {
			b = priority.equals(((PrioritizedItem)obj).getPriority()) 
				 && datum.equals(((PrioritizedItem)obj).getDatum());
		}
		return b;
	}

	public int hashCode() {
		return ( (int)((priority.hashCode() + datum.hashCode()) % Integer.MAX_VALUE) );
	}

	public String toString() {
		String s = "Priority: " + priority + ",    Data: " + datum;
		return s;
	}
}
