import java.util.*;

public class DWhvPriorityQueue extends M865PriorityQueue {
	protected Vector v;
	
	
	public DWhvPriorityQueue(int capacity, int increment) {
		v = new Vector(capacity, increment);
	}
	
	public DWhvPriorityQueue(int capacity) {
		v = new Vector(capacity);
	}
	
	public DWhvPriorityQueue() {
		v = new Vector();
	}
	
	protected DWhvPriorityQueue(Vector v, long h) {
		this.hash = h;
		this.v = (Vector)v.clone();
	}

	protected void bubbleUp(int child) {
		int parent;
		Comparable cPQI, pPQI;
		cPQI = (Comparable)v.elementAt(child);
		while (0 < child) {
			parent = (child-1)/2;
			pPQI = (Comparable)v.elementAt(parent);
			if (pPQI.compareTo(cPQI) <= 0) {
				// The parent has a higher priority than the child, so
				break;
			}
			v.setElementAt(pPQI, child);
			v.setElementAt(cPQI, parent);
			child = parent;
		}
	}
	
	public void enqueue(Comparable pqi) {
	// Add this Comparable to the priority queue.
		hash = (hash + pqi.hashCode()) % Integer.MAX_VALUE;
		v.add( pqi );
		bubbleUp(v.size()-1);
	}
	
	protected void siftDown(int parent) {
		int child, temp;
		Comparable pPQI, cPQI, tPQI;
		pPQI = (Comparable)v.elementAt(parent);
		while (2*parent+1 < v.size()) {
			// There is at least one child
			child = 2*parent + 1;
			cPQI = (Comparable)v.elementAt(child);
			if (child+1 < v.size()) {
				// There is a second child
				temp = child+1;
				tPQI = (Comparable)v.elementAt(temp);
				if (tPQI.compareTo(cPQI) <= 0) {
					// The second child has the higher priority,
					// so swap it with the first child
					child = temp;
					cPQI = tPQI;
				}
			}
			if (pPQI.compareTo(cPQI) <= 0) {
				// The parent has higher priority than the best child, so
				break;
			}
			v.setElementAt(cPQI, parent);
			v.setElementAt(pPQI, child);
			parent = child;
		}
	}
	
	public Comparable dequeue() {
	//  Remove and return a Comparable of highest priority.
	//  Return null if the priority queue is empty.
		if (v.size() <= 0) {
			return null;
		} else if (v.size() == 1) {
			Comparable obj = (Comparable)v.remove(0);
			hash = (hash - obj.hashCode()) % Integer.MAX_VALUE;
			return obj;
		} else {
			Comparable obj = (Comparable)v.get(0);
			hash = (hash - obj.hashCode()) % Integer.MAX_VALUE;
			Comparable pqi = (Comparable)v.remove(v.size()-1);
			v.setElementAt(pqi,0);
			siftDown(0);
			return obj;
		}
	}

	public Comparable peek() {
	//  Return a Comparable of highest priority.
		if (v.isEmpty()) {
			return null;
		} else {
			return ( (Comparable)v.get(0) );
		}
	}
/*
	public void adjustPriority(Comparable obj, Comparable priority) {
	//  Finds the object and adjusts its priority.
		Comparable pqi = null;
		int index = -1;
		for (int i=0; i<v.size(); i++) {
			pqi = (Comparable)v.elementAt(i);
			if (obj.equals(pqi)) {
				index = i;
				break;
			}
		}
		if (0 <= index) {
			boolean higher = pqi.adjustPriority(priority);
			v.setElementAt(pqi, index);
			if ( higher ) {
				// The priority has been raised, so
				bubbleUp(index);
			} else {
				// The priority has been lowered, so
				siftDown(index);
			}
		}
	}
*/
	
	public void clear() {
	// Removes all of the elements from this collection (optional operation). 
		hash = 0;
		v.clear();
	}
		
	public Object clone() {
	// Returns a copy of this queue
		return (new DWhvPriorityQueue(v, hash));
	}

	public boolean contains(Object obj) {
	// Returns true if this collection contains the specified Comparable.
	// This method overrides the corresponding method of the abstract class,
	// because that method uses a DWhvPriorityQueueIterator which is inefficient
	// for this heap-vector implementation.
		boolean b = false;
		for (int i=0; i<v.size(); i++) {
			b = obj.equals( v.get(i) );
			if ( b ) {
				break;
			}
		}
		return b;
	}


	public boolean equals(Object o) {
	// Compares the specified object with this collection for equality. 
		boolean b = (o instanceof M865PriorityQueue) && (this.size() == ((M865PriorityQueue)o).size());
		if (b) {
			Iterator i = this.iterator();
			Iterator j = ((M865PriorityQueue)o).iterator();
			while (b && i.hasNext()) {
				b = (i.next()).equals(j.next());
			}
		}
		return b;
	}
	
	public class DWhvPriorityQueueIterator implements Iterator {
		private DWhvPriorityQueue q, myPQ;
		private long check;
		private Comparable last;
		
		public DWhvPriorityQueueIterator(DWhvPriorityQueue q) {
			this.q = q;
			myPQ = (DWhvPriorityQueue)q.clone();
			check = q.hash;
			last = null;
		}

		public boolean hasNext() {
			//  Check that the queue has not been altered during the processing
			//  of this iterator
			if (check == q.hash) {
				return (0 < myPQ.size());
			} else {
				throw( new ConcurrentModificationException("This DWhvPriorityQueue has been altered"));
			}
		}
		
		public Object next() {
			if (check == q.hash) {
				last = myPQ.dequeue();
				return last;
			} else {
				throw( new ConcurrentModificationException("This DWhvPriorityQueue has been altered"));
			}
		}

		public void remove() {
			// Removes the last element returned by the iterator.
			if (check != q.hash) {
				throw( new ConcurrentModificationException("This DWhvPriorityQueue has been altered"));
			}
			if (last != null) {
				q.remove(last);
			}
		}
	}

	public Iterator iterator() {
	// Returns an iterator over the elements in this collection.  
		return new DWhvPriorityQueueIterator(this);
	}
	
	public boolean remove(Object obj) {
	// Remove the specified object from the priority queue.
		if (!(obj instanceof Comparable)) {
			throw( new ClassCastException("Can only remove Comparables"));
		}
		Comparable pqi = null;
		int index = -1;
		for (int i=0; i<v.size(); i++) {
			pqi = (Comparable)v.elementAt(i);
			if (obj.equals(pqi)) {
				// Found the object, so
				index = i;
				break;
			}
		}
		if (0 <= index) {
			Comparable old = pqi;
			pqi = (Comparable)v.remove(v.size()-1);
			if (index < v.size()) {
				v.setElementAt(pqi, index);
				if (pqi.compareTo(old) <= 0) {
					// The priority has been raised, so
					bubbleUp(index);
				} else {
					// The priority has been lowered, so
					siftDown(index);
				}
			}
			// The priority queue has been changed.
			return true;
		} else {
			// The priority queue is unchanged.
			return false;
		}
	}
	
	public int size() {
	//  Return the number of items in the queue.
		return v.size();
	}

	
	public String toString() {
		String s = super.toString();
		return( "DWhv" + s.substring(4, s.length()) );
	}

	protected String dumpVector() {
		String s = "DWhvPriorityQueue {";
		for (int i=0; i<v.size(); i++) {
			s += "\n " + v.get(i).toString();
		}
		return (s + "\n}");
	}
	
	public static void main(String args[]) {
		DWhvPriorityQueue q1 = new DWhvPriorityQueue();
		String name;
		 
		q1.enqueue(new PrioritizedItem(new Integer(9), "Alice"));
		q1.enqueue(new PrioritizedItem(new Integer(2), "Bob"));
		q1.enqueue(new PrioritizedItem(new Integer(5), "Carol"));
		System.out.println(q1);
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("Empty? " + q1.isEmpty());

		System.out.println("\nq1 Iterator test:");
		Iterator iter = q1.iterator();
		while (iter.hasNext()) {
			name = iter.next().toString();
			System.out.println(name);
		}
		System.out.println("\nLooking at three priority queues");
		DWhvPriorityQueue q2 = new DWhvPriorityQueue();
		q2.addAll(q1);
		DWhvPriorityQueue q3 = (DWhvPriorityQueue)q1.clone();
		System.out.println("q1 equals q2? " + q1.equals(q2));
		System.out.println("q1 equals q3? " + q1.equals(q3));
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("q2.hashCode: " + q2.hashCode());
		System.out.println("q3.hashCode: " + q3.hashCode());
		// System.out.println("Is Bob in q1? " + q1.containsObject("Bob"));
		PrioritizedItem pqi = new PrioritizedItem(new Integer(2), "Bob");
		System.out.println("Is [2, Bob] in q1? " + q1.contains(pqi));
		pqi.setPriority(new Integer(3));
		System.out.println("Is [3, Bob] in q1? " + q1.contains(pqi));
		pqi.setDatum("Doug");
		System.out.println("q1.enqueue: " + pqi);
		q1.enqueue(pqi);
		System.out.println("q1.peek(): " + q1.peek());
		// System.out.println("q1.getPriority(): " + q1.getPriority());
		System.out.println(q1);
		System.out.println("q2.dequeue(): " + q2.dequeue());
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("q2.hashCode: " + q2.hashCode());
		System.out.println("q3.hashCode: " + q3.hashCode());
		System.out.println("q1.size: " + q1.size());
		System.out.println("q2.size: " + q2.size());
		System.out.println("q3.size: " + q3.size());
		System.out.println("q1 equals q2? " + q1.equals(q2));
		System.out.println("q1 equals q3? " + q1.equals(q3));
		System.out.println("Is q1 contained in q2? " + q2.containsAll(q1));
		System.out.println("Is q2 contained in q1? " + q1.containsAll(q2));
		System.out.println("Is q2 contained in q3? " + q3.containsAll(q3));

		System.out.println("q2.dequeue(): " + q2.dequeue());
		System.out.println("q2.dequeue(): " + q2.dequeue());
		System.out.println("q3.clear(): ");
		q3.clear();
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("q2.hashCode: " + q2.hashCode());
		System.out.println("q3.hashCode: " + q3.hashCode());
		System.out.println("q1.size: " + q1.size());
		System.out.println("q2.size: " + q2.size());
		System.out.println("q3.size: " + q3.size());
		System.out.println("q1 equals q2? " + q1.equals(q2));
		System.out.println("q2 equals q3? " + q2.equals(q3));
		
		System.out.println("\nLooking at an array and a priority queue");
		Object[] names = q1.toArray();
		for (int i=0; i<names.length; i++) {
			System.out.println(names[i]);
		}
		PrioritizedItem[] moreNames = (PrioritizedItem[])q1.toArray(new PrioritizedItem[5]);
		System.out.println("moreNames.length: " + moreNames.length);
		for (int i=0; i<moreNames.length; i++) {
			System.out.println(moreNames[i]);
		}
		
		System.out.println("\nTesting the iterator for consistency checking");
		iter = q1.iterator();
		try {
			System.out.println(iter.next());
			System.out.println(iter.next());
			System.out.println("Dequeue: " + q1.dequeue());
			System.out.println(iter.next());
			System.out.println(iter.next());
		} catch (ConcurrentModificationException e) {
			System.out.println("Concurrent Modification Exception: " + e.getMessage());
		}

		System.out.println("\nSort test - on userids");
		String[] students = {
			"Agbor Andu E          AAGBOR   MATH      55",
			"Bagayatkar Prachi P   PBAGAYA  COMP SC   30",
			"Gaertner Nadine       NGAERTN  MATH      30",
			"Saha Snehanshu        SSAHA    MATH      30",
			"Tonkin Charles E      CTONKIN  B&BS      20",
			"Uggirala Ananth       AUGGIRA  I E       30",
			"Whitchurch Dale A     DWHITCH  CP ENGR   30",
			"Woodward Kristen A    KWOODWA  MATH      30",
			"Zhao Meng             MZHAO    MATH      55"};
		PrioritizedItem p;
		for (int i=0; i<students.length; i++) {
			p = new PrioritizedItem(students[i].substring(22, 29), students[i]);
			q3.enqueue(p);
		}
		System.out.println(q3); 
		System.out.println("q3.dumpVector(): " + q3.dumpVector());
	}
}
