import java.io.*;
import java.util.*;

public class EdgeListReader {
	protected TreeSet vertexSet;
	protected TreeSet edgeList;
	protected InputStreamReader inputStream;
	
	public EdgeListReader(InputStreamReader inputStream) {
		this.inputStream = inputStream;
		vertexSet = new TreeSet();
		edgeList = new TreeSet();
	}
			
	public TreeSet getVertexSet() {
		return vertexSet;
	}
	
	public TreeSet getEdgeList() {
		return edgeList;
	}
	
	protected Edge readEdge() 
		throws IOException {
		StringBuffer sourceName = new StringBuffer();
		StringBuffer targetName = new StringBuffer();
		int c;
		char ch;
		int state = 0;

		while ((c = inputStream.read()) != -1) {
			ch = (char)c;
			if (ch == '\n' || ch == '\r') {
				break;
			}
			switch (state) {
				case 0: // Looking for the first letter of the source name
					if (Character.isLetter(ch)) {
						sourceName.append(ch);
						state = 1;
					}
					break;
				case 1: // Looking for more letters or digits in the source name
					if (Character.isLetterOrDigit(ch)) {
						sourceName.append(ch);
					} else {
						state = 2;
					}
					break;
				case 2: // Looking for the first letter of the target name
					if (Character.isLetter(ch)) {
						targetName.append(ch);
						state = 3;
					}
					break;
				case 3: // Looking for more letters or digits in the source name
					if (Character.isLetterOrDigit(ch)) {
						targetName.append(ch);
					} else {
						state = 4;
					}
					break;
				default: // case 4 - scanning to the end of the line.
			}
		}
		if (state == 3 || state == 4) {
			return new Edge(sourceName.toString(), targetName.toString());
		} else if (c == -1) {
			return null;
		} else {
			System.out.println("Invalid edge description.");
			return null;
		}
	}

	public void readEdgeList() 
		throws IOException {
		Edge e;
		while ((e = readEdge()) != null) {
			vertexSet.add(e.getSource());
			vertexSet.add(e.getTarget());
			edgeList.add(e);
		}
	}

	public void dump() {
		int nV = vertexSet.size();
		int nE = edgeList.size();
		System.out.println("There are " + nV + " vertices.");
		System.out.println("The vertices are: ");
		System.out.println(vertexSet);
		System.out.println("There are " + nE + " edges.");
		System.out.println("The edges are: ");
		System.out.println(edgeList);
		if ((nV-1) == nE) {
			System.out.println("This might be a tree.");
		} else {
			System.out.println("This can not be a tree.");
		}
	}
	
	public static void main (String args[])
		throws IOException {

		InputStreamReader inputStream;
		if (0 < args.length) {
			inputStream = new FileReader(args[0]);
		} else {
			inputStream = new InputStreamReader(System.in);
		}
		EdgeListReader reader = new EdgeListReader(inputStream);
		reader.readEdgeList();
		reader.dump();
		/*
		System.out.println("LinkedList check");
		LinkedList l1 = new LinkedList();
		l1.add("Ann");
		l1.add("Barry");
		l1.add("Carol");
		System.out.println(l1);
		LinkedList[] l2 = new LinkedList[2];
		l2[1] = new LinkedList();
		l2[0] = new LinkedList();
		l2[0].add("Ann");
		l2[1].add("Barry");
		l2[0].add("Carol");
		System.out.println(l2[0]);
		System.out.println(l2[1]);
		*/
		M865Graph G = new DWGraph(reader.getVertexSet(), reader.getEdgeList());
		System.out.println("nC: " + G.getNumberOfComponents());
		System.out.println("Connected: " + G.isConnected());
		System.out.println("Has cycles: " + G.hasCycles());
		SortedSet v = reader.getVertexSet();
		Iterator iter = v.iterator();
		while (iter.hasNext()) {
			Comparable vertex = (Comparable)iter.next();
			System.out.println("Degree of "+vertex+" is "+G.degreeOf(vertex));
		}  
   }

}
