import java.io.*;
import java.util.*;

public class M865GraphReader {
	protected InputStreamReader inputStream;
	protected TreeSet vertexSet;
	protected TreeSet edgeList;
	
	public M865GraphReader(String fileName) throws IOException {
		inputStream = new FileReader(fileName);
		vertexSet = new TreeSet();
		edgeList = new TreeSet();
	}
			
	public M865GraphReader() {
		inputStream = new InputStreamReader(System.in);
		vertexSet = new TreeSet();
		edgeList = new TreeSet();
	}

	public M865Graph getGraph() {
		// return new YIGraph(vertexSet, edgeList);
		return new DWGraph(vertexSet, edgeList);
	}

	protected M865Edge readEdge() 
		throws IOException {
		StringBuffer sourceName = new StringBuffer();
		StringBuffer targetName = new StringBuffer();
		int c;
		char ch;
		int state = 0;

		while ((c = inputStream.read()) != -1) {
			ch = (char)c;
			if (ch == '\n' || ch == '\r') {
				break;
			}
			switch (state) {
				case 0: // Looking for the first letter of the source name
					if (Character.isLetter(ch)) {
						sourceName.append(ch);
						state = 1;
					}
					break;
				case 1: // Looking for more letters or digits in the source name
					if (Character.isLetterOrDigit(ch)) {
						sourceName.append(ch);
					} else {
						state = 2;
					}
					break;
				case 2: // Looking for the first letter of the target name
					if (Character.isLetter(ch)) {
						targetName.append(ch);
						state = 3;
					}
					break;
				case 3: // Looking for more letters or digits in the source name
					if (Character.isLetterOrDigit(ch)) {
						targetName.append(ch);
					} else {
						state = 4;
					}
					break;
				default: // case 4 - scanning to the end of the line.
			}
		}
		if (state == 3 || state == 4) {
			M865Vertex source = new M865Vertex(sourceName.toString());
			M865Vertex target = new M865Vertex(targetName.toString());
			return new M865Edge(source,target);
		} else if (c == -1) {
			return null;
		} else {
			System.out.println("Invalid edge description.");
			return null;
		}
	}

	public void readEdgeList() 
		throws IOException {
		M865Edge e;
		while ((e = readEdge()) != null) {
			vertexSet.add(e.getSource());
			vertexSet.add(e.getTarget());
			edgeList.add(e);
		}
	}

	public void dump() {
		int nV = vertexSet.size();
		int nE = edgeList.size();
		System.out.println("There are " + nV + " vertices.");
		System.out.println("The vertices are: ");
		System.out.println(vertexSet);
		System.out.println("There are " + nE + " edges.");
		System.out.println("The edges are: ");
		System.out.println(edgeList);
		if ((nV-1) == nE) {
			System.out.println("This might be a tree.");
		} else {
			System.out.println("This can not be a tree.");
		}
	}
	
	public static void main (String args[])
		throws IOException {

		M865GraphReader graphReader;
		if (0 < args.length) {
			graphReader = new M865GraphReader(args[0]);
		} else {
			graphReader = new M865GraphReader();
		}
		graphReader.readEdgeList();
		graphReader.dump();
		M865Graph g = graphReader.getGraph();
		System.out.println(g);
  }
  

}
