import java.util.*;
import java.lang.reflect.Array;
/**
Class for a Vector Based Queue which implements the Collection Interface.
Used for Math Science 865. More notes to go here.
*/

public class v865Queue extends M865Queue {
// Scope Variables
protected Vector v; // Used to manage Objects stored in Queue
public static final int initCap = 100; // Initial Capacity of Vector
// If not already specified
/************************
* Creates a queue using corresponding values.<BR><BR>
*
* @param capacity Initial capacity of the vector
* @param increment Once full, value to increase vector by<BR><BR>
* @return void
* @author Dr. Warner
************************/
public v865Queue(int capacity, int increment)
{
v = new Vector(capacity, increment);
}
/************************
* Creates a queue setting initial capacity.<BR><BR>
*
* @param capacity Initial capacity of the vector
* @return void
* @author Dr. Warner
************************/
public v865Queue(int capacity)
{
v = new Vector(capacity);
}
/************************
* Creates a queue using default values.<BR><BR>
*
* @return void
* @author Dr. Warner
************************/
public v865Queue()
{
this(initCap);
}
protected v865Queue(Vector v){
this.v = (Vector)v.clone();
}
/*****************
* Will find the first instance of the Object o in the queue.<BR><BR>
*
* @param o Object to be found<BR><BR>
* @return True if at least one of o exists. False otherwise<BR><BR>
* @author Kristen Woodward<BR><BR>
*/
public boolean contains(Object o)
{
return v.contains(o);
}
/*****************
* Add the specific Object to the end of the Queue.
* <BR>
* Pre: s = v.size();
* <BR>
* Post: s + 1 == v.size();
*<BR><BR>
* @param o Object to be added<BR><BR>
* @return True<BR><BR>
* @author Kristen Woodward<BR><BR>
*/
public boolean add(Object o)
{
return v.add(o);
}
/*****************
* Add the specific Collection to the end of the Queue.<BR><BR>
* TODO: Order to be Determined<BR><BR>
*
* Pre: s = v.size();<BR>
*
* Post: s + c.size() == v.size();
*<BR><BR>
* @param c Collection to be added<BR><BR>
* @return True<BR><BR>
* @author Rambabu Maddi<BR><BR>
*/

public boolean addAll(Collection c)
{
return v.addAll(c);
}
/*****************
* Clear the vector
* <BR>
* Post: v.size() == 0;<BR><BR>
*
* @param void<BR><BR>
* @return void<BR><BR>
* @author Rambabu Maddi<BR><BR>
*/
public void clear()
{
v.clear();
}
/*****************
* Returns valid hashcode. Overrides default Object method.
*<BR><BR>
* @param void<BR><BR>
* @return Integer representing hashCode of object.<BR><BR>
* @author Dale Whitchurch<BR><BR>
*/
public int hashCode() {
return v.hashCode();
}

public Iterator iterator() {
   return(new v865QueueIterator(v));
}

/*******************
* Iterator Inner Class
*/
public class v865QueueIterator implements Iterator{
private Vector v;
private int index;
private int check;

public v865QueueIterator(Vector v){
this.v = v;
int index = v.size();
int check = v.hashCode();
}
public boolean hasNext(){
return (v.size() <= index);
}
public Object next(){
return v.get(++ index);
}
public void remove(){
   throw(new UnsupportedOperationException("Ouch"));
}
}
/*****************
* Verify all objects in Collection C are in the queue
*<BR><BR>
* @param C Collection of vectors to be processed<BR><BR>
* @return True if all objects exist at least once in the Queue.<BR><BR>
* @author Rambabu Maddi<BR><BR>
*/
public boolean containsAll(Collection c)
{
return v.containsAll(c);
}
/*****************
* Determines if two Queues are Equal.
* <BR>Note that this simply looks to see if the vectors are equivalent<BR><BR>
*
* @param o Object to check equivalency of<BR><BR>
* @return boolean<BR><BR>
* @author Kristen Woodward
*/
public boolean equals(Object o){
int t = 0;
boolean b = (o instanceof v865Queue) && (((v865Queue)o).size() == this.size());
if (b){
Iterator i = ((v865Queue)o).iterator();
Iterator j = this.iterator();
while (b && j.hasNext()) {
b = (j.next()).equals(i.next());
}
}
return b;
}


/*****************
* Returns an array of Objects in the queue
*<BR><BR>
* @see java.util.Collection#toArray() toArray <BR><BR>
* @param none<BR><BR>
* @return Object[] <BR><BR>
* @author Meng Zhao<BR><BR>
*/
public Object[] toArray()
{
return v.toArray();
}
/*****************
* Generates an array of Objects in the queue
*<BR><BR>
* @param A[] <BR><BR>
* @return Object[]<BR><BR>
* @author
* @throws UnsupportedOperationException Semantics Violation <BR><BR>
*/
public Object[] toArray(Object[] a) {
// Returns an array containing all of the elements in this collection;
//the runtime type of the returned array is that of the specified array.
int i = v.size();
int k = 0;
if (v.size() <= a.length) {
while(0<i){
a[k++] = v.get(--i);
}
if (v.size() != a.length){
a = null;
}
return a;
} else {
Class c = a.getClass();
c = c.getComponentType();
Object[] A = (Object[])Array.newInstance(c, i);
while (0<i){
A[k++] = v.get(--i);
}
return A;
}
}

/*****************
* Check to see if Queue is empty
*<BR><BR>
* @param void<BR><BR>
* @return boolean<BR><BR>
* @author Meng Zhao<BR><BR>
*/
public boolean isEmpty()
{
return v.isEmpty();
}
/*****************
* Check size of Queue
*<BR><BR>
* @param null<BR><BR>
* @return int Size<BR><BR>
* @author Kristen Woodward<BR><BR>
*/
public int size()
{
return v.size();
}
/*****************
* Add an item to the end of the queue
*<BR><BR>
* @param Object o<BR><BR>
* @return void<BR><BR>
* @author Dale Whitchurch<BR><BR>
*/
public void enqueue(Object o)
{
v.add(o);
}
/*****************
* Remove the next item in the queue and return it
*<BR><BR>
* @param null<BR><BR>
* @return Object<BR><BR>
* @author Dale Whitchurch<BR><BR>
*/
public Object dequeue()
{
Object o = v.firstElement();
v.remove(0);
return o;
}
/*****************
* Return a reference to the first object in the queue
*<BR><BR>
* @param null<BR><BR>
* @return Object<BR><BR>
* @author Dale Whitchurch<BR><BR>
*/
public Object peek()
{
return v.firstElement();
}
public String toString(){
String s = "v865Queue {";
Iterator k = this.iterator();
}
public Object clone(){
return (new v865Queue(v));
}

public static void main(String args[]) {
v865Queue q1 = new v865Queue();
String name;

q1.enqueue("Alice");
q1.add("Bob");
q1.enqueue("Carol");
System.out.println(q1);
System.out.println("q1.hashCode: " + q1.hashCode());
System.out.println("Empty? " + q1.isEmpty());
System.out.println("\nq1 Iterator test:");
Iterator iter = q1.iterator();
while (iter.hasNext()) {
name = (String)iter.next();
System.out.println(name);
}

System.out.println("\nLooking at three queues");
v865Queue q2 = new v865Queue();
q2.addAll(q1);
v865Queue q3 = (v865Queue)q1.clone();
System.out.println("q1 equals q2? " + q1.equals(q2));
System.out.println("q1 equals q3? " + q1.equals(q3));
System.out.println("q1.hashCode: " + q1.hashCode());
System.out.println("q2.hashCode: " + q2.hashCode());
System.out.println("q3.hashCode: " + q3.hashCode());
System.out.println("Is Bob in q1? " + q1.contains("Bob"));
q1.add("Doug");
System.out.println("q1.peek(): " + q1.peek());
System.out.println(q1);
System.out.println("q2.dequeue(): " + q2.dequeue());
System.out.println("q1.hashCode: " + q1.hashCode());
System.out.println("q2.hashCode: " + q2.hashCode());
System.out.println("q3.hashCode: " + q3.hashCode());
System.out.println("q1.size: " + q1.size());
System.out.println("q2.size: " + q2.size());
System.out.println("q3.size: " + q3.size());
System.out.println("q1 equals q2? " + q1.equals(q2));
System.out.println("q1 equals q3? " + q1.equals(q3));
System.out.println("Is q1 contained in q2? " + q2.containsAll(q1));
System.out.println("Is q2 contained in q1? " + q1.containsAll(q2));
System.out.println("Is q2 contained in q3? " + q3.containsAll(q3));
System.out.println("q2.dequeue(): " + q2.dequeue());
System.out.println("q2.dequeue(): " + q2.dequeue());
q3.clear();
System.out.println("q1.hashCode: " + q1.hashCode());
System.out.println("q2.hashCode: " + q2.hashCode());
System.out.println("q3.hashCode: " + q3.hashCode());
System.out.println("q1.size: " + q1.size());
System.out.println("q2.size: " + q2.size());
System.out.println("q3.size: " + q3.size());
System.out.println("q1 equals q2? " + q1.equals(q2));
System.out.println("q2 equals q3? " + q2.equals(q3));

System.out.println("\nLooking at an array and a queue");
Object[] names = q1.toArray();
for (int i=0; i<names.length; i++) {
System.out.println(names[i]);
}
}
}
