import java.io.*;

public class XMLScanner {

	public static final int EOF = 0;
	public static final int STARTTAG = 1;
	public static final int ENDTAG = 2;
	public static final int SINGLETAG = 3;
	public static final int MISC = 4;
	public static final int BADTAG = 5;
	
	public static final int LT = 1;
	public static final int SLASH = 2;
	public static final int GT = 3;
	public static final int WS = 4;
	public static final int NL = 5;
	
	public int token;
	public String scannedString;
	public String tag;
	
	protected InputStreamReader fr;
	protected int state = 0;
	protected int charClass[];
	protected int f[][] =	{{ 1,  2,  1,  1,  1,  0},
							 { 1,  2,  1,  1,  1,  0},
							 { 4, 12,  3, 12,  4, 12},
							 { 5, 12, 12, 12,  5, 12},
							 { 4, 12,  8,  9,  6, 12},
							 { 5, 12, 12, 11,  7, 12},
							 { 6, 12,  8,  9,  6, 12},
							 { 7, 12, 12, 11,  7, 12},
							 {12, 12, 12, 10, 12, 12},
							 { 1,  2,  1,  1,  1,  0},
							 { 1,  2,  1,  1,  1,  0},
							 { 1,  2,  1,  1,  1,  0},
							 {12, 12, 12, 12, 12, 12}};
	protected StringBuffer newString;
	protected StringBuffer newTag;
	protected int c;
	
	public XMLScanner( InputStreamReader fr) {
		this.fr = fr;
		token = EOF;
		scannedString = "";
		tag = "";
		
		charClass = new int[256];
		charClass['<']  = LT;
		charClass['/']  = SLASH;
		charClass['>']  = GT;
		charClass[' ']  = WS;
		charClass['\t'] = WS;
		charClass['\r'] = NL;
		charClass['\n'] = NL;
		
		newString = new StringBuffer("");
		newTag = new StringBuffer("");
		
	}
	
	private void flush() {
		scannedString = (newString.toString()).trim();
		newString = new StringBuffer();
	}

	private void flush1() {
		flush();
		newString.append((char)c);
	}

	private void flush2() {
		newString.append((char)c);
		flush();
		tag = newTag.toString();
		newTag = new StringBuffer();

	}
		
	public int scan() 
		throws IOException {

		while ((c = fr.read()) != -1) {
			// System.out.println("State: " + state + "   char: " + (char)c + "   class: " + charClass[c]);
			state = f[state][charClass[c]];
			// Perform the output action
			switch(state) {
				case 0:  // We have seen a new line
					flush();
					if (scannedString.length() > 0) {
						return (token = MISC);
					}
					break;
				case 1:  // We are building a MISC string
					newString.append((char)c);
					break;
				case 2:  // We just saw a LT 
					flush1();
					newTag = new StringBuffer();
					if (scannedString.length() > 0) {
						return (token = MISC);
					}
					break;
				case 3:  // We just saw a LT SLASH
					newString.append((char)c);
					break;
				case 4:  // We are building a STARTTAG or a SINGLETAG
				case 5:  // We are building an ENDTAG
					newString.append((char)c);
					newTag.append((char)c);
					break;
				case 6:	
				case 7:  // We are building the attribute string
					newString.append((char)c);
					break;
				case 8:  // We have just seen a SLASH this should be an SINGLETAG
					newString.append((char)c);
					break;
				case 9:  // We have just seen the GT for a STARTTAG
					flush2();
					return (token = STARTTAG);
				case 10: // We have just seen the GT for a SINGLETAG
					flush2();
					return (token = SINGLETAG);
				case 11: // We have just seen the GT for an ENDTAG
					flush2();
					return (token = ENDTAG);	
				case 12:
					return (token = BADTAG);
			}
		}
		return (token = EOF);
	}							
}

		
	